<?php
/**
Saves and retrieves gallery items from XML files.

This file could be adapted to manage gallery information in a MySQL database
instead, but for simplicity XML files are used.
**/
class GalData 
{
  public $key;
  public $person;
  public $email;
  public $doby;
  public $dobm;
  public $dobd;
  public $loc;
  public $desc;
  public $imgw;
  public $imgh;
  public $tnw;
  public $tnh;
  public $date;

  private $folder;
  private $doc;

  public function __construct($folder) 
  {
    $this->folder = $folder;
  }

  public function infofile()
  {
    return $this->folder.date('Ymd', $this->date).'_'.$this->key.'.xml';
  }

  public function imgfile()
  {
    return $this->folder.date('Ymd', $this->date).'_'.$this->key.'.jpeg';
  }
  
  public function tnfile()
  {
    return $this->folder.date('Ymd', $this->date).'_'.$this->key.'.tn.jpeg';
  }

  public function dob()
  {
    return $this->dobd.'/'.$this->dobm.'/'.$this->doby;
  }

  public function age()
  {
    $y1 = $this->doby;
    if (!$y1) return false;
    $m1 = $this->dobm;
    if (!$m1) $m1 = 6;
    $d1 = $this->dobd;
    if (!$d1) $d1 = 15;

    $y2 = idate('Y', $this->date);
    $m2 = idate('m', $this->date);
    $d2 = idate('d', $this->date);

    $age = $y2 - $y1;
    if ($m1 > $m2 || $m1 == $m2 && $d1 > $d2) $age--;
    return ($age < 0) ? false : $age;
  }

  private function writeField($item, $fld, $val)
  {
    $el = $this->doc->createElement($fld);
    $el->appendChild($this->doc->createTextNode($val));
    $item->appendChild($el);
  }

  /* 
  Save the gallery info. Must set date and key before using.
  File name is created from appropriate directory, date, and key.
  File name is returned.
  */
  public function save()
  {
    $this->doc = new DOMDocument();
    $this->doc->formatOutput = true;
    $item = $this->doc->createElement('item');
    $this->doc->appendChild($item); 
    $this->writeField($item, FN_KEY, $this->key);
    $this->writeField($item, FN_DATE, $this->date);
    $this->writeField($item, FN_PERSON, $this->person);
    $this->writeField($item, FN_EMAIL, $this->email);
    $this->writeField($item, FN_DOBY, $this->doby);
    $this->writeField($item, FN_DOBM, $this->dobm);
    $this->writeField($item, FN_DOBD, $this->dobd);
    $this->writeField($item, FN_LOC, $this->loc);
    $this->writeField($item, FN_DESC, $this->desc);
    $this->writeField($item, FN_IMGW, $this->imgw);
    $this->writeField($item, FN_IMGH, $this->imgh);
    $this->writeField($item, FN_TNW, $this->tnw);
    $this->writeField($item, FN_TNH, $this->tnh);
    $fn = $this->infofile();
    $this->doc->save($fn);
    return $fn;
  }

  private function readField($item, $fld)
  {
    $el = $item->getElementsByTagName($fld)->item(0);
    return $el->nodeValue;
  }

  /* 
  Load the gallery info based on current date and key.
  Set date and key before using.
  Returns true on success, false on failure.
  */
  public function load()
  {
    return $this->loadFromFile($this->infofile());
  }

  /* 
  Load gallery info from file name.
  No need to set date and key as these are obtained from file.
  Returns true on success, false on failure.
  */
  public function loadFromFile($fn)
  {
    if (!file_exists($fn)) return false;
    $this->doc = new DOMDocument();
    $this->doc->load($fn);
    $item = $this->doc->getElementsByTagName('item')->item(0);
    $this->key = $this->readField($item, FN_KEY);
    $this->date = $this->readField($item, FN_DATE);
    $this->person = $this->readField($item, FN_PERSON);
    $this->email = $this->readField($item, FN_EMAIL);
    $this->doby = $this->readField($item, FN_DOBY);
    $this->dobm = $this->readField($item, FN_DOBM);
    $this->dobd = $this->readField($item, FN_DOBD);
    $this->loc = $this->readField($item, FN_LOC);
    $this->desc = $this->readField($item, FN_DESC);
    $this->imgw = $this->readField($item, FN_IMGW);
    $this->imgh = $this->readField($item, FN_IMGH);
    $this->tnw = $this->readField($item, FN_TNW);
    $this->tnh = $this->readField($item, FN_TNH);
    return true;
  }

  private function moveFile($src, $dst)
  {
    if (file_exists($src)) {
      return rename($src, $dst);
    } else {
      return false;
    }
  }

  public function moveImagesFrom($imgfile, $tnfile)
  {
    $r1 = $this->moveFile($imgfile, $this->imgfile());
    $r2 = $this->moveFile($tnfile, $this->tnfile());
    return $r1 && $r2;
  }

  /*
  Save changes and move from queue to gallery.
  */
  public function moveTo($folder)
  {
    // Get current file names
    $oldinfo = $this->infofile();
    $oldimg = $this->imgfile();
    $oldtn = $this->tnfile();
    // Change queue flag so that file names will use live folder
    $this->folder = $folder;
    // Move files
    $r1 = $this->moveFile($oldinfo, $this->infofile());
    $r2 = $this->moveFile($oldimg, $this->imgfile());
    $r3 = $this->moveFile($oldtn, $this->tnfile());
    return $r1 && $r2 && $r3;
  }

  public function sendModEmail()
  {
    $msg = 'New gallery picture...'.NL
      .selfPath().'/'.'galmod.php?fn='.$this->infofile().'&secret='.urlencode(GAL_SECRET);
    mail(MOD_EMAIL, MOD_SUBJECT, $msg);
  }

  public function toString()
  {
    return $this->doc->saveXML();
  }

  private static function getlist($dir, $newestfirst, $max)
  {
    $files = glob($dir.'*.xml');
    if ($newestfirst) {
      $files = array_reverse($files);
    }
    if (count($files) > $max) {
      $files = array_slice($files, 0, $max);
    }
    return $files;
  }

  public static function showqueue()
  {
    $files = GalData::getlist(DIR_QUEUE, false, MAX_QUEUE);
    if (count($files) == 0) {
      echo '<p>No pictures to display.</p>';
      return;
    }
    foreach ($files as $fn) {
      $tn = str_replace('.xml', '.tn.jpeg', $fn);
      echo '<div class="tnqueue">'
        .'<a href="galmod.php?fn='.$fn.'&secret='.urlencode(GAL_SECRET).'">'
        .'<img src="'.$tn.'"></a></div>';
    }
  }

  private function nameAndAge()
  {
    $age = $this->age();
    $ret = $this->person;
    if ($age !== false) {
      $ret .= ', '.$age.'y.o.';
    }
    return $ret;
  }

  public function display()
  {
    echo '<img src="'.$this->imgfile().'">';
    echo '<p class="disp"><b>by:</b> '.$this->person.'</p>';
    $age = $this->age();
    if ($age !== false) {
      echo '<p class="disp"><b>age:</b> '.$this->age.' y.o.</p>';
    }
    if ($this->loc) {
      echo '<p class="disp"><b>from:</b> '.$this->loc.'</p>';
    }
    if ($this->desc) {
      echo '<p class="disp"><b>comment:</b><br/>'.nl2br($this->desc).'</p>';
    }
  }

  public static function showgallery()
  {
    $files = GalData::getlist(DIR_PIC, true, MAX_GALLERY);
    if (count($files) == 0) {
      echo '<p>No pictures to display.</p>';
      return;
    }
    foreach ($files as $fn) {
      $gal = new GalData(DIR_PIC);
      $gal->loadFromFile($fn);
      $tn = $gal->tnfile();
      echo '<div class="tngallery">'
        .'<a href="gallery.php?fn='.$fn.'">'
        .'<img src="'.$tn.'"></a>';
      echo '<p>'.$gal->nameAndAge().'</p>';
      if ($gal->loc) {
        echo '<p>'.$gal->loc.'</p>';
      }
      echo '</div>';
    }
  }

}